# Generated by Django 2.2.16 on 2021-06-24 15:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pipeline", "0030_auto_20210607_1210"),
    ]

    operations = [
        migrations.AddField(
            model_name="pipelineinstance",
            name="is_expired",
            field=models.BooleanField(default=False, help_text="运行时被定期清理即为过期", verbose_name="是否已经过期"),
        ),
        migrations.AlterField(
            model_name="pipelineinstance",
            name="create_time",
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name="创建时间"),
        ),
    ]
